"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executePromiseWithTimeout = void 0;
const errors_1 = require("../errors");
const utils_1 = require("./utils");
/**
 * Execute a function that returns a promise, passing a `resolve` and
 * `reject` functions to it.
 *
 * Works like the `new Promise` constructor, but encapsulates it with a timeout.
 *
 * @example
 *
 * function example(shouldPass: boolean) {
 *   const items: string[] = [];
 *   items.push("item 1");
 *   return executePromiseWithTimeout<string[]>((resolve, reject) => {
 *      if(shouldPass) {
 *         items.push("item 2");
 *         resolve(items);
 *      } else {
 *         reject();
 *      }
 *   }, 1000, items);
 * }
 *
 * // If the functions reachs the timeout, it will throw the
 * // exception TimeoutError with the items array as data property
 *
 * @throws TimeoutError if the function reachs timeout. This error has a property
 * data that is passed throw the `rejectedData` parameter
 *
 * @param fn function to be executed.
 * @param timeout Time to wait for completion of the function.
 * @param rejectedData Data that should be added in rejection if the function reachs timeout.
 * @returns Return of `fn` parameter.
 * @internal
 */
function executePromiseWithTimeout(fn, timeout, rejectedData) {
    if (!fn) {
        throw new Error("can not execute an null function");
    }
    if (utils_1.utils.isInDebugMode()) {
        return new Promise((resolve, reject) => {
            fn((value) => {
                resolve(value);
            }, reject);
        });
    }
    return new Promise((resolve, reject) => {
        const nodeTimeout = setTimeout(() => {
            reject(new errors_1.TimeoutError("timeout", rejectedData));
        }, timeout);
        fn((value) => {
            clearTimeout(nodeTimeout);
            resolve(value);
        }, reject);
    });
}
exports.executePromiseWithTimeout = executePromiseWithTimeout;
