"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getStackTrace = void 0;
const consts_1 = require("../consts");
/**
 * Gets the trace of the current function / object
 * Spacing each line of the trace with `DEFAULT_STACK_TRACE_LIMIT`
 *
 * @example
 *
 * 20  function creator(fn) {
 * 21    fn();
 * 22    return getStackTrace();
 * 23  }
 * 24
 * 25  creator(() => {});
 *
 * // Being that the function is in file: D:\github\corde\test.js
 * // And current working dir: D:\github\corde\test.js
 *
 * // Outputs:
 *     creator (test.js:22:10)
 *     Object.<anonymous> (test.js:25:13)
 *
 * @param stackLimit Amount of lines to be collected.
 * @returns Formated stack trace.
 *
 * @internal
 */
function getStackTrace(stackLimit, removeFirstStack = true, functionName) {
    const obj = {};
    Error.prepareStackTrace = (_, calls) => {
        const stacksWithoutFirstCall = calls.slice(removeFirstStack ? 1 : 0);
        // There is no need of 100% stack trace,
        // so we remove irrelevant paths.
        const formatFunctionName = functionName ? `${functionName} ` : "";
        const trace = consts_1.EXPECT_RECEIVED_TAB_SPACE +
            "at " +
            formatFunctionName +
            stacksWithoutFirstCall
                .filter((s) => isStrackRelevant(s))
                .slice(0, stackLimit ?? consts_1.DEFAULT_STACK_TRACE_LIMIT)
                .join("\n" + consts_1.EXPECT_RECEIVED_TAB_SPACE + "at ");
        // removes full path of the file for security.
        const formatedPath = process.cwd().replace(/\\/g, "\\\\");
        const regex = new RegExp(formatedPath + "\\\\", "g");
        return trace.replace(regex, "");
    };
    Error.captureStackTrace(obj, getStackTrace);
    return obj.stack;
}
exports.getStackTrace = getStackTrace;
function isStrackRelevant(stack) {
    const fileName = stack.getFileName();
    const functionName = stack.getFunctionName();
    return (!stack.isNative() &&
        !stack.isEval() &&
        (!functionName || !functionName.includes("Promise")) &&
        (!fileName ||
            (!fileName.startsWith("internal") &&
                !fileName.includes("\node_modules") &&
                !fileName.includes("node_modules"))));
}
